package cn.gyee.nxdp.services;

import cn.gyee.nxdp.CacheContext;
import cn.gyee.nxdp.Contant;
import cn.gyee.nxdp.entities.auto.WindTurbineTestingPointAi2;
import cn.gyee.nxdp.entities.auto.Windturbine;
import cn.gyee.nxdp.entities.nxdp.WindturbineTarget;
import cn.gyee.nxdp.entities.vo.DataVo;
import cn.gyee.nxdp.util.ObjUtil;
import cn.gyee.nxdp.util.golden.EdosUtil;
import cn.gyee.nxdp.util.golden.IEdosUtil;
import cn.gyee.nxdp.util.redis.RedisService;
import com.alibaba.fastjson.JSONArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

/**
 * @ClassName WindturbineTargetService
 * @Description TODO
 * @Author 谢生杰
 * @Date 2020/9/18 9:28
 * @Version 1.0
 **/
@Service
public class WindturbineTargetService {
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private RedisService redisService;

    public void wtToRedis() throws Exception {
        IEdosUtil edosUtil = new EdosUtil();
        List<Windturbine> wtls = CacheContext.wtls;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        for (Windturbine wt : wtls) {
            WindturbineTarget windturbineTarget = new WindturbineTarget();
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.MILLISECOND, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            String formatDate = sdf.format(calendar.getTime());
            String sql = "select maximumtem,minimumtem,weather from weatherfd where windpowerstationid=? and to_char(recodedata,'yyyy-MM-dd')=?";
            List<DataVo> tqlist = jdbcTemplate.query(sql, new RowMapper<DataVo>() {
                @Override
                public DataVo mapRow(ResultSet rs, int i) throws SQLException {
                    DataVo vo = new DataVo();
                    String tq = rs.getString("weather");
                    Double maximumtem = rs.getDouble("maximumtem");
                    Double minimumtem = rs.getDouble("minimumtem");
                    Double wd = (maximumtem + minimumtem) / 2;
                    vo.setSvalue1(tq);
                    vo.setDvalue1(wd);
                    return vo;
                }
            }, wt.getWindpowerstationid(),formatDate);
            if (tqlist != null && tqlist.size() > 0) {
                DataVo vo = tqlist.get(0);
                windturbineTarget.setTq(vo.getSvalue1() == null ? "" : vo.getSvalue1());//天气
                windturbineTarget.setWd(vo.getDvalue1() == null ? 0 : vo.getDvalue1());//温度
                windturbineTarget.setKqmd(101325 / (287.05 * (vo.getDvalue1() + 273.15)));//空气密度(kg/m3)
            }
            Map<String, WindTurbineTestingPointAi2> ai2Map = CacheContext.wtmap.get(wt.getId());

            double pjfs = edosUtil.getRealData(ai2Map.get(Contant.JSSSFS).getId()).getPointValueInDouble();
            windturbineTarget.setFl(ObjUtil.getFlByFs(pjfs)); //风力

            String mmTemp = getWtYmtbf(wt.getId());
            String[] mmArr = mmTemp.split("_");
            windturbineTarget.setYmtbf(Double.parseDouble(mmArr[0]));//月mtbf
            double ysbklyl = getYsbklyl(wt.getId());
            windturbineTarget.setYsbklyl(ysbklyl);//月设备可利用率



            redisService.set(wt.getId(),JSONArray.toJSON(windturbineTarget).toString());
        }

    }

    private double getYsbklyl(String wtid) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");

        Calendar cal = Calendar.getInstance();

        String rq = sdf.format(cal.getTime());

        int day = cal.get(Calendar.DAY_OF_MONTH);

        // 风机数
//        List<Windturbine> wtls = CacheContext.wtls;
//        int fjnum = wtls.size();
        DataVo vo = queryStopDataCurrentMonth(rq,wtid);
        double gzsj = vo.getDvalue1();

        return (day * 24  - gzsj) / (day * 24 );
    }

    private String getWtYmtbf(String wtid) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");

        Calendar cal = Calendar.getInstance();

        String rq = sdf.format(cal.getTime());

        int day = cal.get(Calendar.DAY_OF_MONTH);


        DataVo vo = queryStopDataCurrentMonth(rq,wtid);
        int count = vo.getIvalue1();
        double gzsj = vo.getDvalue1();

        // 风机数
//        List<Windturbine> wtls = CacheContext.wtls;
//        int fjnum = wtls.size();

        // 日历小时
        double dayhours = day * 24;

        double mtbf = count == 0 ? 0 : dayhours  / count;// 平均间隔时间
        double mttr = count == 0 ? 0 : gzsj / count;

        return mtbf + "_" + mttr;
    }

    private DataVo queryStopDataCurrentMonth(String rq,String wtid) {
        StringBuilder sb = new StringBuilder();
        sb.append("select sum(cs) cs,sum(sssj) sj from MTTRMTBF_FJ where rq='");
        sb.append(rq).append("' and windturbineid='").append(wtid).append("'");
        List<DataVo> list = jdbcTemplate.query(String.valueOf(sb),
                new RowMapper<DataVo>() {

                    @Override
                    public DataVo mapRow(ResultSet rs, int arg1)
                            throws SQLException {
                        DataVo vo = new DataVo();
                        vo.setIvalue1(rs.getInt("cs"));
                        vo.setDvalue1(rs.getDouble("sj"));
                        return vo;
                    }

                });
        if (list != null && list.size() > 0) {
            return list.get(0);
        }
        return null;
    }
}
