package cn.gyee.nxdp.services.early;

import cn.gyee.nxdp.entities.vo.WtEarlyVo;
import cn.gyee.nxdp.model.auto.Earlywarningknowledgemain;
import cn.gyee.nxdp.model.auto.Earlywarningknowledgesub;
import cn.gyee.nxdp.model.auto.Windturbine;
import cn.gyee.nxdp.service.IEarlywarningknowledgemainService;
import cn.gyee.nxdp.service.IEarlywarningknowledgesubService;
import cn.gyee.nxdp.service.IWindturbineService;
import cn.gyee.nxdp.util.JdbcUtils;
import cn.gyee.nxdp.util.common.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName : EarlyWarningService
 * @Author : xieshengjie
 * @Date: 2021/1/12 23:15
 * @Description : 预警分析反馈
 */
@Service
public class EarlyWarningService {

    private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    @Autowired
    private JdbcTemplate operate;
    @Autowired
    private IEarlywarningknowledgemainService earlywarningknowledgemainService;
    @Autowired
    private IEarlywarningknowledgesubService earlywarningknowledgesubService;
    @Autowired
    private IWindturbineService windturbineService;

    public Map<String, Map<String,Integer>> findWtEarlyCount(String beginDate, String endDate, String wpid, String modelid, List<Earlywarningknowledgesub> selectRules) {
        Map<String, Map<String,Integer>> resultMap = new HashMap<>();
       StringBuilder sb = new StringBuilder();
       sb.append("select b.windturbineid,b.alerttext,sum(a.count) count from alarmcount a  left join  alarmsnap b on a.snapid=b.id where 1=1 ");
       if (!StringUtils.isEmpty(beginDate)) {
           sb.append(" and to_char(a.alarmdate,'yyyy-mm-dd')>='").append(beginDate).append("'");
       }
        if (!StringUtils.isEmpty(endDate)) {
            sb.append(" and to_char(a.alarmdate,'yyyy-mm-dd')<='").append(endDate).append("'");
        }
        if (!StringUtils.isEmpty(wpid)) {
            sb.append(" and b.stationid='").append(wpid).append("'");
        }
        if (!StringUtils.isEmpty(modelid)) {
            sb.append(" and b.modelid='").append(modelid).append("'");
        }
        if (!StringUtils.isEmpty(selectRules)) {
            sb.append(" and (");
            for (Earlywarningknowledgesub rule : selectRules) {
                if (!StringUtils.isEmpty(rule.getRule())) {
                    if (!StringUtils.isEmpty(rule.getMark()) && !StringUtils.isEmpty(rule.getAmount() + "")) {
                        sb.append("(b.alerttext='").append(rule.getRule()).append("' and a.count")
                                .append(rule.getMark()).append(rule.getAmount()).append(") and");
                    } else {
                        sb.append("(b.alerttext='").append(rule.getRule()).append("' and a.count>0) and");
                    }
                }
            }
            sb.deleteCharAt(sb.length()-1);
            sb.deleteCharAt(sb.length()-1);
            sb.deleteCharAt(sb.length()-1);
            sb.append(")");
        }


        sb.append(" group by b.windturbineid,b.alerttext");
       List<WtEarlyVo> list = operate.query(String.valueOf(sb), new RowMapper<WtEarlyVo>() {
           @Override
           public WtEarlyVo mapRow(ResultSet rs, int i) throws SQLException {
               WtEarlyVo vo = new WtEarlyVo();
               vo.setWtid(rs.getString("windturbineid"));
               vo.setRule(rs.getString("alerttext"));
               vo.setCount(rs.getInt("count"));
               return vo;
           }
       });
        List<String> wtList = list.stream().map(i -> i.getWtid()).distinct().collect(Collectors.toList());
        if (wtList!=null && wtList.size()>0) {
            for (String wtid : wtList) {
                List<WtEarlyVo> collect = list.stream().filter(i -> i.getWtid().equals(wtid)).collect(Collectors.toList());
                Map<String,Integer> subMap = new HashMap<>();
                for (WtEarlyVo wtEarlyVo : collect) {
                    subMap.put(wtEarlyVo.getRule(),wtEarlyVo.getCount());
                }
                resultMap.put(wtid,subMap);
            }
            return resultMap;
        }
        return null;
    }

    public void saveEarlyWarning(String wpid,String modelid,List<Earlywarningknowledgesub> selectRules) {
        Earlywarningknowledgemain main = new Earlywarningknowledgemain();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        main.setModelname(wpid+"_"+modelid+"_"+sdf.format(new Date()));
        main.setWindpowerstationid(wpid);
        main.setModel(modelid);
        main.setCreatedate(new Date());
        earlywarningknowledgemainService.save(main);
        Long tid = main.getId();

        for (Earlywarningknowledgesub rule : selectRules) {
            Earlywarningknowledgesub sub = new Earlywarningknowledgesub();
            sub.setTid(tid);
            sub.setRule(rule.getRule());
            sub.setMark(rule.getMark());
            sub.setAmount(rule.getAmount());
            earlywarningknowledgesubService.save(sub);
        }

    }

    public List<Earlywarningknowledgemain> findDataByWpAndModel(String wpid,String modelid) {
        return earlywarningknowledgemainService.findDataByWpAndModel(wpid,modelid);
    }

    public List<Earlywarningknowledgesub> findSubListByMain(Integer tid) {
        return earlywarningknowledgesubService.getByTid(tid);
    }

    public void createWobugeq(String wt, String description,Map<String,Windturbine> sismap,Map<String,String[]> mismap)  {


        Connection conn = null;
        Statement st = null;
        ResultSet rs = null;
        try {
            conn = JdbcUtils.getConnection(); // 获取数据库连接
            st = conn.createStatement();  //获得SQL的执行对象
            StringBuilder sb = new StringBuilder();
            sb.append(
                    "INSERT INTO Recommenmain ( WTID, RECODEDATE, CREATEDATE, SPEED, REASON, ISMAIN, WOBUGID, OPERATION, OPERATIONDATE, REMARK, BAK1, BAK2,  WPID, WTNUM, WPNUM,RID) VALUES(");
            sb.append("'").append(wt).append("'");
            sb.append(",'").append(format.format(new Date())).append("'");
            sb.append(",'").append(format.format(new Date())).append("'");
            sb.append(",").append(0);
            sb.append(",'").append(description).append("'");
            sb.append(",'").append("是").append("'");
            sb.append(",null");// WOBUGID
            sb.append(",null");// OPERATION
            sb.append(",null");// OPERATIONDATE
            sb.append(",null");// REMARK
            sb.append(",null");// BAK1
            sb.append(",null");// BAK2
            sb.append(",'").append(sismap.get(wt).getWindpowerstationid()).append("'");
            String[] mis = mismap.get(wt);
            sb.append(",").append(mis[0]);
            sb.append(",").append(mis[1]);
            String rid = UUID.randomUUID().toString().replaceAll("\\-","");
            sb.append(",'").append(rid).append("')");
            String sql = String.valueOf(sb);
            int i = st.executeUpdate(sql);
            if (i>0){
                System.out.println("插入成功");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }finally {
            JdbcUtils.relese(conn,st,rs);
        }
    }

    public Map<String,String[]> getMismap(){
        Map<String,String[]> mismap = new HashMap<>();

        Connection conn = null;
        Statement st = null;
        ResultSet rs = null;
        try {
            conn = JdbcUtils.getConnection(); // 获取数据库连接
            st = conn.createStatement();  //获得SQL的执行对象
            String sql = "select wtid,wtnum,ownerwf from WINDTURBINE";
            rs = st.executeQuery(sql);
            while(rs.next()) {
                // int getInt(int columnIndex) 根据列索引取列的数据
                // int getInt(String columnLabel) 根据列名取列的数据
                String[] mis = new String[2];
                String id = rs.getString("wtid");
                mis[0]= rs.getString("wtnum");
                mis[1] = rs.getString("ownerwf");

                mismap.put(id,mis);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }finally {
            JdbcUtils.relese(conn,st,rs);
        }
        return mismap;
    }
}
