package cn.gyee.nxdp.services.shbracelet;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import cn.gyee.nxdp.entities.custom.PointData;
import cn.gyee.nxdp.entities.vo.PersonLnglat;
import cn.gyee.nxdp.entities.vo.Shbracelet;
import cn.gyee.nxdp.entities.vo.ShbraceletVo;
import cn.gyee.nxdp.model.auto.Shbruser;
import cn.gyee.nxdp.service.IShbraceletpointkeyService;
import cn.gyee.nxdp.service.IShbraceletuserService;
import cn.gyee.nxdp.service.IShbruserService;
import cn.gyee.nxdp.util.GpsCorrect;
import cn.gyee.nxdp.util.golden.EdosUtil;
import cn.gyee.nxdp.util.golden.IEdosUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;


@Service
public class ShbraceletService {


    @Autowired
    private JdbcTemplate operate;
    @Autowired
    private IShbruserService shbruserService;
    @Autowired
    private IShbraceletuserService shbraceletuserService;
    @Autowired
    private IShbraceletpointkeyService shbraceletpointkeyService;

    private IEdosUtil ednaApiUtil = new EdosUtil();

    public List<ShbraceletVo> findShbraceletData(String userid, Date beginDate,
                                                 Date endDate) throws Exception {
        List<ShbraceletVo> voList = new ArrayList<ShbraceletVo>();
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        long diff = endDate.getTime() - beginDate.getTime();
        long hour = diff % nd / nh;
        long min = diff % nd % nh / nm;
        List<String> points = findShbraceletpointsByUserid(userid);
        if (points != null && points.size() > 0) {
            List<PointData> pointTemp = ednaApiUtil.getHistoryDatasSnap(
                    points.get(0), beginDate.getTime() / 1000,
                    endDate.getTime() / 1000,
                    min / 10, 600l);
            for (PointData pointData : pointTemp) {
                ShbraceletVo vo = new ShbraceletVo();
                vo.setDate(pointData.getPointTime() * 1000);
                voList.add(vo);
            }
        }
        for (String point : points) {
            try {
                List<PointData> pointDataList = ednaApiUtil
                        .getHistoryDatasSnap(point, beginDate.getTime() / 1000,
                                endDate.getTime() / 1000, min / 10, 600l);
                for (int i = 0; i < pointDataList.size(); i++) {
                    PointData pointData = pointDataList.get(i);

                    if (pointData.getEdnaId().contains("PLD")) {
                        voList.get(i).setPld(pointData.getPointValueInDouble());
                    } else if (pointData.getEdnaId().contains("SSXY")) {
                        voList.get(i)
                                .setSsxy(pointData.getPointValueInDouble());
                    } else if (pointData.getEdnaId().contains("SZXY")) {
                        voList.get(i)
                                .setSzxy(pointData.getPointValueInDouble());
                    } else if (pointData.getEdnaId().contains("XTPL")) {
                        voList.get(i)
                                .setXtpl(pointData.getPointValueInDouble());
                    } else if (pointData.getEdnaId().contains("XYND")) {
                        voList.get(i).setXy(pointData.getPointValueInDouble());
                    } else {
                        continue;
                    }

                }
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }



        return voList;
    }

    private List<String> findShbraceletpointsByUserid(String userid) {
        String sql = "select pointkey from shbraceletpointkey where deviceid=(select deviceid from shbraceletuser where userid=?)";
        List<String> pointList = operate.query(sql, new RowMapper<String>() {

            @Override
            public String mapRow(ResultSet rs, int arg1) throws SQLException {

                return rs.getString("pointkey");
            }

        }, userid);
        if (pointList != null && pointList.size() > 0) {
            return pointList;
        }

        return null;
    }

    public List<PersonLnglat> findPersonLnglatData() throws Exception {
        List<PersonLnglat> plList = new ArrayList<>();
        String sql = "select u.username from shbruser u,shbraceletuser us where u.id=us.userid";
        List<String> nameList = operate.query(sql, new RowMapper<String>() {
            @Override
            public String mapRow(ResultSet rs, int i) throws SQLException {
                return rs.getString("username");
            }
        });
        for (int i = 0; i < nameList.size(); i++) {
            PersonLnglat pl = new PersonLnglat();
            String sql1 = "select sk.pointkey from shbruser u,shbraceletuser us,shbraceletpointkey sk where u.id= us.userid and us.deviceid=sk.deviceid and (sk.pointkey like '%LNG' or sk.pointkey like '%LAT') and u.username=?";
            pl.setLnglat(new Double[2]);
            List<String> lngLatList = operate.query(sql1, new RowMapper<String>() {
                @Override
                public String mapRow(ResultSet rs, int i) throws SQLException {
                    return rs.getString("pointkey");
                }
            }, nameList.get(i));
            for (String lngLat : lngLatList) {
                Double[] lnglatArr = pl.getLnglat();
                if (lngLat.contains("LNG")) {
                    lnglatArr[0] = ednaApiUtil.getRealData(lngLat).getPointValueInDouble();
                } else {
                    lnglatArr[1] = ednaApiUtil.getRealData(lngLat).getPointValueInDouble();
                }
            }

//            pl.setId(i+1);
            pl.setName(nameList.get(i));
            pl.setState(1);
            plList.add(pl);
        }
        List<PersonLnglat> resultList = plList.stream().filter(c -> c.getLnglat()[0] != 0.0).collect(Collectors.toList());
        
        for (int i =0;i<resultList.size();i++){

            Double[] lnglat = resultList.get(i).getLnglat();

            resultList.get(i).setLnglat(GpsCorrect.transform(lnglat[0],lnglat[1]));
            resultList.get(i).setId(i+1);
        }
        
        return resultList;
    }

    public List<Shbracelet> findShbraceletDataNew() throws Exception {
        List<Shbracelet> resultList = new ArrayList<>();
        List<Shbruser> userlist = shbruserService.list();
        for(Shbruser user : userlist){
            Shbracelet shbracelet = new Shbracelet();
            Integer deviceid = shbraceletuserService.getDeviceidByUserid(user.getId());
            List<String> pointkeyList = shbraceletpointkeyService.getPointkeylistByDeviceid(deviceid);
            List<PointData> pointDataList = ednaApiUtil.getRealData(pointkeyList);
            Integer isrun = 0;
            toShbracelet(shbracelet,pointDataList);
            shbracelet.setUserid(user.getId());
            shbracelet.setUsername(user.getUsername());
            //shbracelet.setIsrun(isrun);
            resultList.add(shbracelet);
        }
        List<Shbracelet> collect = resultList.stream().filter(c ->
                c.getLng() != null && c.getLng() != 0.0)
                .collect(Collectors.toList());
        for(Shbracelet sh : collect){
            Double[] transform = GpsCorrect.transform(sh.getLng(), sh.getLat());
            sh.setLng(transform[0]);
            sh.setLat(transform[1]);
        }
        return collect;
    }

    private void toShbracelet(Shbracelet shbracelet, List<PointData> pointDataList) {
        long currentTime = System.currentTimeMillis();
        for(PointData pointData : pointDataList) {
            String ednaId = pointData.getEdnaId();
            double dataValue = pointData.getPointValueInDouble();
            Long pointTime = pointData.getPointTime();
            long s = (currentTime - pointTime) / (60 * 1000);
            shbracelet.setIsrun(s < 10 ? 1 : 0);//1代表动，0代表不动

            if (ednaId.contains("BS")){
                shbracelet.setBs(dataValue);
            }else if(ednaId.contains("LNG")) {
                shbracelet.setLng(dataValue);
            }else if(ednaId.contains("LAT")) {
                shbracelet.setLat(dataValue);
            }else if(ednaId.contains("PLD")) {
                if (dataValue>80){
                    shbracelet.setIsrun(2);
                }
                shbracelet.setPld(dataValue);
            }else if(ednaId.contains("SSXY")) {
                if (dataValue<70 || dataValue>89){
                    shbracelet.setIsrun(2);
                }
                shbracelet.setSsxy(dataValue);
            }else if(ednaId.contains("SZXY")) {
                if (dataValue<110 || dataValue>139){
                    shbracelet.setIsrun(2);
                }
                shbracelet.setSzxy(dataValue);
            }else if(ednaId.contains("XTPL")) {
                if (dataValue<60 || dataValue>100){
                    shbracelet.setIsrun(2);
                }
                shbracelet.setXtpl(dataValue);
            }else if(ednaId.contains("XYND")) {
                if (dataValue<96 || dataValue>100){
                    shbracelet.setIsrun(2);
                }
                shbracelet.setXynd(dataValue);
            }
        }
    }
}
