package cn.gyee.nxdp.websocket;

import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @ClassName CompanyWebSocket
 * @Description TODO
 * @Author 谢生杰
 * @Date 2020/9/2 10:13
 * @Version 1.0
 **/
@ServerEndpoint(value = "/windturbine/{wtid}")
@Component
public class WindturbineWebSocket {
    /**
     * 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
     */
    private static int onlineCount = 0;

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     * 在外部可以获取此连接的所有websocket对象，并能对其触发消息发送功能，我们的定时发送核心功能的实现在与此变量
     */
    private static ConcurrentHashMap<String, WindturbineWebSocket> webSocketSet = new ConcurrentHashMap<>();

    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;


    private  String wtid;
    /**
     * 连接建立成功调用的方法
     * <p>
     * 类似dwr的onpage方法，参考之前文章中demo有
     */
    @OnOpen
    public void onOpen(@PathParam("wtid") String wtid, Session session) {
        this.session = session;
        this.wtid = wtid;
        webSocketSet.put(wtid,this);    //加入set中
        addOnlineCount();           //在线数加1
        System.out.println("有新连接加入！当前在线人数为" + getOnlineCount());
        try {
            sendMessage("连接已建立成功.");
        } catch (Exception e) {
            System.out.println("IO异常");
        }
    }

    /**
     * 连接关闭调用的方法
     * <p>
     * 参考dwrsession摧毁方法
     */
    @OnClose
    public void onClose() {
        webSocketSet.remove(this);  //连接关闭后，将此websocket从set中删除
        subOnlineCount();           //在线数减1
        System.out.println("有一连接关闭！当前在线人数为" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        System.out.println("来自客户端的消息:" + message);
    }

    // 错误提示
    @OnError
    public void onError(Session session, Throwable error) {
        System.out.println("发生错误");
        error.printStackTrace();
    }

    // 发送消息，在定时任务中会调用此方法
    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);

    }


    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WindturbineWebSocket.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        WindturbineWebSocket.onlineCount--;
    }


    public Session getSession() {
        return session;
    }

    public void setSession(Session session) {
        this.session = session;
    }

    public static ConcurrentHashMap<String, WindturbineWebSocket> getWebSocketSet() {
        return webSocketSet;
    }
    public void setConcurrentHashMap(ConcurrentHashMap webSocketSet) {
        this.webSocketSet = webSocketSet;
    }
}
